package net.sf.saxon.dom;
import net.sf.saxon.value.SequenceExtent;
import org.w3c.dom.Node;

/**
* This class wraps a SequenceExtent as a DOM NodeList
*  - though this will only work if the items are nodes, and if the nodes themselves
* implement the DOM Node interface (which is true of the two Saxon tree models,
* but not necessarily of all possible implementations).
*/

public final class DOMNodeList implements org.w3c.dom.NodeList {
    private SequenceExtent sequence;

    /**
    * Construct an node list that wraps a supplied SequenceExtent
    */

    public DOMNodeList(SequenceExtent extent) {
        sequence = extent;
    }

    /**
    * return the number of nodes in the list (DOM method)
    */

    public int getLength() {
        return sequence.getLength();
    }

    /**
    * Return the n'th item in the list (DOM method)
    * @throws java.lang.ClassCastException if the item is not a DOM Node
    */

    public Node item(int index) {
        return (Node)sequence.itemAt(index);
    }

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//

