package net.sf.saxon.event;
import javax.xml.transform.TransformerException;

/**
 * This class is a filter that passes all Receiver events through unchanged,
 * except that it removes all type annotations from element and attribute nodes
 *
 * @author           Michael Kay (michael.h.kay@ntlworld.com)
 */


public class AnnotationRemover extends ProxyReceiver {

    /**
     * Output element start tag
     */

    public void startElement(int nameCode, int typeCode, int properties) throws TransformerException {
        super.startElement(nameCode, 0, properties);
    }

    /**
     * Handle an attribute
     *
     * @param            nameCode integer identifying the name of the attribute
     * @param            typeCode integer identifying the type annotation of the
     *                   attribute
     * @param            value the value of the attribute
     * @param            properties bit-significant flags indicating special action to
     *                   be taken
     * @throws           TransformerException On any failure to write the attribute
     */

    public void attribute(int nameCode, int typeCode, CharSequence value, int properties)
    throws TransformerException {
        super.attribute(nameCode, 0, value, properties);
    }

};

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of Software AG (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//

