package net.sf.saxon.event;
import net.sf.saxon.om.NamePool;
import net.sf.saxon.om.Item;
import net.sf.saxon.om.NodeInfo;
import net.sf.saxon.om.DocumentInfo;
import net.sf.saxon.om.SequenceIterator;
import net.sf.saxon.om.Axis;
import net.sf.saxon.pattern.AnyNodeTest;
import net.sf.saxon.value.AtomicValue;
import org.xml.sax.Locator;
import java.util.Properties;
import javax.xml.transform.TransformerException;

/**
  * This class allows output to be generated. It channels output requests to an
  * Emitter which does the actual writing. This is an abstract class, there are
  * concrete implementions for XML output and text output.
  *
  * @author <A HREF="mailto:michael.h.kay@ntlworld.com>Michael H. Kay</A>
  */

public abstract class Outputter implements SequenceReceiver {

    //protected Receiver receiver;
    protected boolean previousAtomic = false;

    public void setNamePool(NamePool pool) {}

    public NamePool getNamePool() {
        // it doesn't matter what we return, as this outputter never handles names
        return NamePool.getDefaultNamePool();
    }

    public void setSystemId(String systemId) {}

    public String getSystemId() {
        return null;
    }

    public void setDocumentLocator(Locator locator) {}

    public void setUnparsedEntity(String name, String systemId, String publicId) {}


    //public abstract Properties getOutputProperties();

    /**
    * Start the output process
    */

    public void startDocument() throws TransformerException {
        //System.err.println("Open " + this + " using emitter " + emitter.getClass());
        //receiver.startDocument();
        previousAtomic = false;
    }



}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
