package net.sf.saxon.event;


/**
  * ReceiverOptions defines a set of constants, which can be used in
  * calls to methods on the Receiver interface. The values are
  * bit-significant.
  *
  * @author Michael H. Kay (michael.h.kay@ntlworld.com)
  */


public class ReceiverOptions {

    /**
    * Flag to disable output escaping
    */

    public final static int DISABLE_ESCAPING = 1;

    /**
    * Flag indicating that the value has already been checked and the serializer
    * does not need to check it further
    */

    public final static int VALUE_CHECKED = 2;

    /**
    * Flag indicating that the value contains no special characters
    * that need to be escaped
    */

    public final static int NO_SPECIAL_CHARS = 4;

    /**
    * Flag indicating that the name has been checked and is known to be valid
    */

    public final static int NAME_OK = 8;

    /**
    * Flag indicating that the name consists entirely of ASCII characters
    */

    public final static int NAME_IS_ASCII = 16;

    /**
    * Flag indicating that duplicate values should be rejected
    */

    public final static int REJECT_DUPLICATES = 32;

    /**
     * Flag indicating that the namespace (of an element or attribute name)
     * has already been declared; it does not need to be generated by the namespace
     * fixup process.
     */

    public final static int NAMESPACE_OK = 64;

    /**
     * Flag passed on startElement indicating that the element does not inherit
     * the namespaces of its ancestors.
     */

    public final static int DISINHERIT_NAMESPACES = 128;

    /**
     * Flag used when an attribute value or text node contains null characters
     * before and after strings generated by character mapping; these strings
     * are to be output without escaping
     */

    public final static int USE_NULL_MARKERS = 256;

    /**
     * Flag indicating that an element or attribute annotated as an xs:QName or
     * xs:NOTATION has been lexically checked but still requires a check that the
     * prefix is in scope
     */

    public final static int NEEDS_PREFIX_CHECK = 512;

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file, other than fragments copied from the SAX distribution
// made available by David Megginson, and the line marked PB-SYNC.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// The line marked PB-SYNC is by Peter Bryant (pbryant@bigfoot.com). All Rights Reserved.
//
// Contributor(s): Michael Kay, Peter Bryant, David Megginson
//
