package net.sf.saxon.event;
import net.sf.saxon.event.Builder;
import net.sf.saxon.event.ContentEmitter;
import net.sf.saxon.event.Stripper;
import net.sf.saxon.om.DocumentInfo;
import net.sf.saxon.Controller;

import javax.xml.transform.*;
import javax.xml.transform.sax.TransformerHandler;

import org.xml.sax.*;


/**
  * <b>TransformerReceiver</b> is similar in concept to the JAXP TransformerHandler,
 * except that it implements Saxon's Receiver interface rather than the standard
 * SAX2 interface. This means that it allows nodes with type annotations to be
 * passed down a pipeline from one transformation to another.
  */

public class TransformerReceiver extends ProxyReceiver {

    Controller controller;
    Builder builder;
    Result result;
    String systemId;

    /**
    * Create a TransformerHandlerImpl and initialise variables. The constructor is protected, because
    * the Filter should be created using newTransformerReceiver() in the SAXTransformerFactory
    * class
    */

    public TransformerReceiver(Controller controller) {
        this.controller = controller;
        setNamePool(controller.getNamePool());
        builder = controller.makeBuilder();
        builder.setNamePool(controller.getNamePool());
        Stripper stripper = controller.makeStripper(builder);
        this.setUnderlyingReceiver(stripper);
    }

    // TODO: make the TransformerReceiver serially reusable

    /**
    * Get the Transformer used for this transformation
    */

    public Transformer getTransformer() {
        return controller;
    }

    /**
    * Set the SystemId of the document
    */

    public void setSystemId(String url) {
        systemId = url;
        builder.setSystemId(url);
    }

    /**
    * Get the systemId of the document
    */

    public String getSystemId() {
        return systemId;
    }

    /**
     * Notify the start of an element
     * @param nameCode integer code identifying the name of the element within the name pool.
     * @param typeCode integer code identifying the element's type within the name pool.
     * @param properties: for future use. Should be set to zero.
     */

    public void startElement(int nameCode, int typeCode, int properties) throws TransformerException {
        super.startElement(nameCode, typeCode, properties);
    }

    /**
    * Set the output destination of the transformation
    */

    public void setResult(Result result) {
        if (result==null) {
            throw new IllegalArgumentException("Result must not be null");
        }
        this.result = result;
    }

    /**
    * Get the output destination of the transformation
    */

    public Result getResult() {
        return result;
    }

    /**
    * Override the behaviour of endDocument() in ContentEmitter, so that it fires off
    * the transformation of the constructed document
    */

    public void endDocument() throws TransformerException {
        super.endDocument();
        DocumentInfo doc = builder.getCurrentDocument();
        if (doc==null) {
            throw new TransformerException("No source document has been built");
        }
        doc.getNamePool().allocateDocumentNumber(doc);
        controller.transformDocument(doc, result);
    }




}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): None
//
