package net.sf.saxon.instruct;
import net.sf.saxon.Controller;
import net.sf.saxon.ParameterSet;
import net.sf.saxon.om.NamePool;
import net.sf.saxon.expr.XPathContext;
import net.sf.saxon.value.SequenceType;

import javax.xml.transform.TransformerException;

/**
* An xsl:apply-imports element in the stylesheet
*/

public class ApplyImports extends Instruction {

    private WithParam[] actualParams = null;
    private WithParam[] tunnelParams = null;

    public ApplyImports( WithParam[] actualParams,
                         WithParam[] tunnelParams ) {
        this.actualParams = actualParams;
        this.tunnelParams = tunnelParams;
    }


    /**
    * Get the name of this instruction for diagnostic and tracing purposes
    * @return the string "xsl:apply-imports"
    */


    public void setInstructionDetails(Controller controller, NamePool namePool, InstructionDetails details) {
        details.setInstructionName("apply-imports");
    }

    public SequenceType getResultType() {
        return SequenceType.ANY_SEQUENCE;
    }

    public TailCall processLeavingTail(XPathContext context) throws TransformerException {

        Controller controller = context.getController();

        // handle parameters if any

        ParameterSet params = assembleParams(context, actualParams);
        ParameterSet tunnels = assembleTunnelParams(context, tunnelParams);

        Template currentTemplate = context.getController().getCurrentTemplate();
        if (currentTemplate==null) {
            throw new TransformerException("There is no current template rule");
        }

        int min = currentTemplate.getMinImportPrecedence();
        int max = currentTemplate.getPrecedence()-1;
        controller.applyImports(min, max, params, tunnels);
        return null;
                // We never treat apply-imports as a tail call, though we could
    }


}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
