package net.sf.saxon.instruct;
import net.sf.saxon.Controller;
import net.sf.saxon.om.NamePool;
import net.sf.saxon.expr.XPathContext;

import javax.xml.transform.TransformerException;

/**
* This instruction throws an error which was detected at compile time but is not to be
* raised unless the instruction is actually executed.
*/

public class DeferredError extends Instruction {

    private TransformerException error;
    private String instructionName;

    public DeferredError(TransformerException error, String instructionName) {
        if (error==null) {
            throw new NullPointerException("Deferred Error: no error value supplied");
        }
        this.error = error;
        this.instructionName = instructionName;
    }

    public void setInstructionDetails(Controller controller, NamePool namePool, InstructionDetails details) {
        details.setInstructionName(instructionName);
    }

    public TailCall processLeavingTail(XPathContext context) throws TransformerException {
        throw error;
    }
}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s):
// Portions marked "e.g." are from Edwin Glaser (edwin@pannenleiter.de)
//
