package net.sf.saxon.instruct;
import net.sf.saxon.expr.XPathContext;
import net.sf.saxon.value.Value;
import net.sf.saxon.Controller;
import net.sf.saxon.om.NamePool;
import net.sf.saxon.om.SequenceIterator;

import javax.xml.transform.TransformerException;

/**
* Handler for xsl:variable elements in stylesheet. <br>
* The xsl:variable element has mandatory attribute name and optional attribute select
*/

public class Variable extends DefiningVariable {

    public void setInstructionDetails(Controller controller, NamePool namePool, InstructionDetails details) {
        details.setInstructionName("variable");
        super.setInstructionDetails(controller, namePool, details);
    }

    /**
    * Process the variable declaration
    */

    public TailCall processLeavingTail(XPathContext context) throws TransformerException {
        Bindery b = context.getController().getBindery();
        if (isGlobal()) {
            if (!b.isEvaluated(this)) {                 // don't evaluate a global variable twice
                Value value = getSelectValue(context);
                b.defineGlobalVariable(this, value);
            }
        } else {
            Value value = getSelectValue(context);
            // System.err.println("value of variable = " + value.getClass() + " = " + value);
            // value.display(10);
            b.setLocalVariable(getSlotNumber(), value);
        }
        return null;
    }

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
