package net.sf.saxon.om;
import net.sf.saxon.expr.LastPositionFinder;

import java.util.List;

/**
* Class ArrayListIterator, iterates over a sequence of items held in a Java ArrayList,
* or indeed in any other kind of List
*/

final public class ArrayListIterator
        implements AxisIterator, LastPositionFinder, LookaheadIterator {

    int index=0;
    int length;
    Item current = null;
    List list = null;

    public ArrayListIterator(List list) {
        index = 0;
        this.list = list;
        this.length = list.size();
    }

    public boolean hasNext() {
        return index<length;
    }

    public Item next() {
        if (index >= length) return null;
        current = (Item)list.get(index++);
        return current;
    }

    public Item current() {
        return current;
    }

    public int position() {
        return index;
    }

    public int getLastPosition() {
        return length;
    }

    public SequenceIterator getAnother() {
        return new ArrayListIterator(list);
    }

    /**
     * Indicate that any nodes returned in the sequence will be atomized. This
     * means that if it wishes to do so, the implementation can return the typed
     * values of the nodes rather than the nodes themselves. The implementation
     * is free to ignore this hint.
     * @param atomizing true if the caller of this iterator will atomize any
     * nodes that are returned, and is therefore willing to accept the typed
     * value of the nodes instead of the nodes themselves.
     */

    //public void setIsAtomizing(boolean atomizing) {}

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//

