package net.sf.saxon.style;
import net.sf.saxon.om.NamespaceConstant;

import java.util.HashMap;

public abstract class StandardNames {

	/**
	 * Well-known names used in XSLT processing. These names must all have
     * fingerprints in the range 0-1023, to avoid clashing with codes allocated
     * in a NamePool. We use the top three bits for the namespace, and the bottom
     * seven bits for the local name
	*/

    private final static int DFLT_NS = 0;
    private final static int XSL_NS = 1;
    private final static int SAXON_NS = 2;
    private final static int XML_NS = 3;
    private final static int XS_NS = 4;
    private final static int XDT_NS = 5;
    private final static int XSI_NS = 6;

    public final static int DFLT = 128*0;
    public final static int XSL = 128*1;
    public final static int SAXON = 128*2;
    public final static int XML = 128*3;
    public final static int XS = 128*4;
    public final static int XDT = 128*5;
    public final static int XSI = 128*6;

    public final static int XSL_ANALYZE_STRING = XSL + 0;
    public final static int XSL_APPLY_IMPORTS = XSL + 1;
    public final static int XSL_APPLY_TEMPLATES = XSL + 2;
    public final static int XSL_ATTRIBUTE = XSL + 3;
    public final static int XSL_ATTRIBUTE_SET = XSL + 4;
    public final static int XSL_CALL_TEMPLATE = XSL + 5;
    public final static int XSL_CHARACTER_MAP = XSL + 6;
    public final static int XSL_CHOOSE = XSL + 7;
    public final static int XSL_COMMENT = XSL + 10;
    public final static int XSL_COPY = XSL + 11;
    public final static int XSL_COPY_OF = XSL + 12;
    public final static int XSL_DECIMAL_FORMAT = XSL + 13;
    public static final int XSL_DOCUMENT = XSL + 14;
    public final static int XSL_ELEMENT = XSL + 15;
    public final static int XSL_FALLBACK = XSL + 16;
    public final static int XSL_FOR_EACH = XSL + 17;
    public final static int XSL_FOR_EACH_GROUP = XSL + 20;
    public final static int XSL_FUNCTION = XSL + 21;
    public final static int XSL_IF = XSL + 22;
    public final static int XSL_IMPORT = XSL + 23;
    public final static int XSL_IMPORT_SCHEMA = XSL + 24;
    public final static int XSL_INCLUDE = XSL + 25;
    public final static int XSL_KEY = XSL + 26;
    public final static int XSL_MATCHING_SUBSTRING = XSL + 27;
    public final static int XSL_MESSAGE = XSL + 30;
    public final static int XSL_NEXT_MATCH = XSL + 31;
    public final static int XSL_NUMBER = XSL + 32;
    public final static int XSL_NAMESPACE = XSL + 33;
    public final static int XSL_NAMESPACE_ALIAS = XSL + 34;
    public final static int XSL_NON_MATCHING_SUBSTRING = XSL + 35;
    public final static int XSL_OTHERWISE = XSL + 36;
    public final static int XSL_OUTPUT = XSL + 37;
    public final static int XSL_OUTPUT_CHARACTER = XSL + 41;
    public final static int XSL_PARAM = XSL + 42;
    public final static int XSL_PERFORM_SORT = XSL + 43;
    public final static int XSL_PRESERVE_SPACE = XSL + 44;
    public final static int XSL_PROCESSING_INSTRUCTION = XSL + 45;
    public final static int XSL_RESULT_DOCUMENT = XSL + 46;
    public final static int XSL_SEQUENCE = XSL + 47;
    public final static int XSL_SORT = XSL + 50;
    public final static int XSL_STRIP_SPACE = XSL + 51;
    public final static int XSL_STYLESHEET = XSL + 52;
    public final static int XSL_TEMPLATE = XSL + 53;
    public final static int XSL_TEXT = XSL + 54;
    public final static int XSL_TRANSFORM = XSL + 55;
    public final static int XSL_VALUE_OF = XSL + 56;
    public final static int XSL_VARIABLE = XSL + 57;
    public final static int XSL_WITH_PARAM = XSL + 60;
    public final static int XSL_WHEN = XSL + 61;

    public final static int XSL_XPATH_DEFAULT_NAMESPACE = XSL + 100;
    public final static int XSL_EXCLUDE_RESULT_PREFIXES = XSL + 101;
    public final static int XSL_EXTENSION_ELEMENT_PREFIXES = XSL + 102;
    public final static int XSL_TYPE = XSL + 103;
    public final static int XSL_USE_ATTRIBUTE_SETS = XSL + 104;
    public final static int XSL_VALIDATION = XSL + 105;
    public final static int XSL_VERSION = XSL + 106;

    private final static String XSL_B = "{" + NamespaceConstant.XSLT + "}";
    public final static String XSL_VERSION_CLARK =
            XSL_B + "version";
    public final static String XSL_XPATH_DEFAULT_NAMESPACE_CLARK =
            XSL_B + "xpath-default-namespace";
    public final static String XSL_EXTENSION_ELEMENT_PREFIXES_CLARK =
            XSL_B + "extension-element-prefixes";
    public final static String XSL_EXCLUDE_RESULT_PREFIXES_CLARK =
            XSL_B + "exclude-result-prefixes";




    public final static int SAXON_ASSIGN = SAXON + 1;
    public final static int SAXON_CALL_TEMPLATE = SAXON + 2;
    public final static int SAXON_COLLATION = SAXON + 3;
    public final static int SAXON_DOCTYPE = SAXON + 4;
    public final static int SAXON_ENTITY_REF = SAXON + 5;
    public final static int SAXON_IMPORT_QUERY = SAXON + 6;
    public final static int SAXON_SCRIPT = SAXON + 7;
    public final static int SAXON_WHILE = SAXON + 8;

    private static final String SAXON_B = "{" + NamespaceConstant.SAXON + "}";
    public final static String SAXON_ASSIGNABLE = SAXON_B + "assignable";
    public final static String SAXON_CHARACTER_REPRESENTATION = SAXON_B + "character-representation";
    public final static String SAXON_EXPLAIN = SAXON_B + "explain";
    public final static String SAXON_INDENT_SPACES = SAXON_B + "indent-spaces";
    public final static String SAXON_NEXT_IN_CHAIN = SAXON_B + "next-in-chain";
    public final static String SAXON_BYTE_ORDER_MARK = SAXON_B + "byte-order-mark";
    public final static String SAXON_REQUIRE_WELL_FORMED = SAXON_B + "require-well-formed";
    public final static String SAXON_MEMO_FUNCTION = SAXON_B + "memo-function";

    // Not sure about this one...
    public static final int SAXON_JAVA_LANG_OBJECT = SAXON + 29;

    public static final int XML_BASE = XML + 1;
    public static final int XML_SPACE = XML + 2;
    public static final int XML_LANG = XML + 3;

//    public final static int ARCHIVE = DFLT + 0;
//    public final static int AS = DFLT + 1;
//    public final static int CASE_ORDER = DFLT + 2;
//    public final static int CDATA_SECTION_ELEMENTS = DFLT + 3;
//    public final static int CHARACTER = DFLT + 4;
//    public final static int CLASS = DFLT + 5;
//    public final static int COLLATION = DFLT + 6;
//    public final static int COPY_NAMESPACES = DFLT + 7;
//    public final static int COUNT = DFLT + 10;
//    public final static int DATA_TYPE = DFLT + 11;
//    public final static int DECIMAL_SEPARATOR = DFLT + 12;
//    public final static int DECOMPOSITION = DFLT + 13;
//    public final static int DEFAULT = DFLT + 14;
//    public final static int DEFAULT_VALIDATION = DFLT + 15;
//    public final static int DIGIT = DFLT + 16;
//    public final static int DISABLE_OUTPUT_ESCAPING = DFLT + 17;
//    public final static int DOCTYPE_PUBLIC = DFLT + 18;
//    public final static int DOCTYPE_SYSTEM = DFLT + 20;
//    public final static int ELEMENTS = DFLT + 21;
//    public final static int ESCAPE_URI_ATTRIBUTES = DFLT + 22;
//    public final static int ENCODING = DFLT + 23;
//    public final static int EXCLUDE_RESULT_PREFIXES = DFLT + 24;
//    public final static int EXTENSION_ELEMENT_PREFIXES = DFLT + 25;
//    public final static int FLAGS = DFLT + 26;
//    public final static int FORMAT = DFLT + 27;
//    public final static int FROM = DFLT + 30;
//    public final static int GROUP_ADJACENT = DFLT + 31;
//    public final static int GROUP_BY = DFLT + 32;
//    public final static int GROUP_ENDING_WITH = DFLT + 33;
//    public final static int GROUP_STARTING_WITH = DFLT + 34;
//    public final static int GROUPING_SEPARATOR = DFLT + 35;
//    public final static int GROUPING_SIZE = DFLT + 36;
//    public final static int HREF = DFLT + 37;
//    public final static int ID = DFLT + 40;
//    public final static int IMPLEMENTS_PREFIX = DFLT + 41;
//    public final static int INCLUDE_CONTENT_TYPE = DFLT + 42;
//    public final static int INDENT = DFLT + 43;
//    public final static int INFINITY = DFLT + 44;
//    public final static int LANG = DFLT + 45;
//    public final static int LANGUAGE = DFLT + 46;
//    public final static int LETTER_VALUE = DFLT + 47;
//    public final static int LEVEL = DFLT + 50;
//    public final static int MATCH = DFLT + 51;
//    public final static int MEDIA_TYPE = DFLT + 52;
//    public final static int METHOD = DFLT + 53;
//    public final static int MINUS_SIGN = DFLT + 54;
//    public final static int MODE = DFLT + 55;
//    public final static int NAME = DFLT + 56;
//    public final static int NAMESPACE = DFLT + 57;
//    public final static int NAN = DFLT + 60;
//    public final static int OMIT_XML_DECLARATION = DFLT + 61;
//    public final static int ORDER = DFLT + 62;
//    public final static int ORDINAL = DFLT + 63;
//    public final static int OVERRIDE = DFLT + 64;
//    public final static int PATTERN_SEPARATOR = DFLT + 65;
//    public final static int PERCENT = DFLT + 66;
//    public final static int PER_MILLE = DFLT + 67;
//    public final static int PRIORITY = DFLT + 70;
//    public final static int REGEX = DFLT + 71;
//    public final static int REQUIRED = DFLT + 72;
//    public final static int RESULT_PREFIX = DFLT + 73;
//    public final static int RULES = DFLT + 74;
//    public final static int SCHEMA_LOCATION = DFLT + 75;
//    public final static int SELECT = DFLT + 76;
//    public final static int SEPARATOR = DFLT + 77;
//    public final static int SRC = DFLT + 80;
//    public final static int STANDALONE = DFLT + 81;
//    public final static int STRENGTH = DFLT + 82;
//    public final static int STRING = DFLT + 83;
//    public final static int STYLESHEET_PREFIX = DFLT + 84;
//    public final static int TERMINATE = DFLT + 85;
//    public final static int TEST = DFLT + 86;
//    public final static int TUNNEL = DFLT + 87;
//    public final static int TYPE = DFLT + 90;
//    public final static int UNDECLARE_NAMESPACES = DFLT + 91;
//    public final static int USE = DFLT + 92;
//    public final static int USE_ATTRIBUTE_SETS = DFLT + 93;
//    public final static int USE_CHARACTER_MAPS = DFLT + 94;
//    public final static int VALIDATION = DFLT + 95;
//    public final static int VALUE = DFLT + 96;
//    public final static int VERSION = DFLT + 97;
//    public final static int XPATH_DEFAULT_NAMESPACE = DFLT + 100;
//    public final static int ZERO_DIGIT = DFLT + 101;

    public final static String ARCHIVE = "archive";
    public final static String AS = "as";
    public final static String CASE_ORDER = "case-order";
    public final static String CDATA_SECTION_ELEMENTS = "cdata-section-elements";
    public final static String CHARACTER = "character";
    public final static String CLASS = "class";
    public final static String COLLATION = "collation";
    public final static String COPY_NAMESPACES = "copy-namespaces";
    public final static String COUNT = "count";
    public final static String DATA_TYPE = "data-type";
    public final static String DECIMAL_SEPARATOR = "decimal-separator";
    public final static String DECOMPOSITION = "decomposition";
    public final static String DEFAULT = "default";
    public final static String DEFAULT_VALIDATION = "default-validation";
    public final static String DIGIT = "digit";
    public final static String DISABLE_OUTPUT_ESCAPING = "disable-output-escaping";
    public final static String DOCTYPE_PUBLIC = "doctype-public";
    public final static String DOCTYPE_SYSTEM = "doctype-system";
    public final static String ELEMENTS = "elements";
    public final static String ESCAPE_URI_ATTRIBUTES = "escape-uri-attributes";
    public final static String ENCODING = "encoding";
    public final static String EXCLUDE_RESULT_PREFIXES = "exclude-result-prefixes";
    public final static String EXTENSION_ELEMENT_PREFIXES = "extension-element-prefixes";
    public final static String FLAGS = "flags";
    public final static String FORMAT = "format";
    public final static String FROM = "from";
    public final static String GROUP_ADJACENT = "group-adjacent";
    public final static String GROUP_BY = "group-by";
    public final static String GROUP_ENDING_WITH = "group-ending-with";
    public final static String GROUP_STARTING_WITH = "group-starting-with";
    public final static String GROUPING_SEPARATOR = "grouping-separator";
    public final static String GROUPING_SIZE = "grouping-size";
    public final static String HREF = "href";
    public final static String ID = "id";
    public final static String IMPLEMENTS_PREFIX = "implements-prefix";
    public final static String INCLUDE_CONTENT_TYPE = "include-content-type";
    public final static String INDENT = "indent";
    public final static String INFINITY = "infinity";
    public final static String LANG = "lang";
    public final static String LANGUAGE = "language";
    public final static String LETTER_VALUE = "letter-value";
    public final static String LEVEL = "level";
    public final static String MATCH = "match";
    public final static String MEDIA_TYPE = "media-type";
    public final static String METHOD = "method";
    public final static String MINUS_SIGN = "minus-sign";
    public final static String MODE = "mode";
    public final static String NAME = "name";
    public final static String NAMESPACE = "namespace";
    public final static String NAN = "NaN";
    public final static String OMIT_XML_DECLARATION = "omit-xml-declaration";
    public final static String ORDER = "order";
    public final static String ORDINAL = "ordinal";
    public final static String OVERRIDE = "override";
    public final static String PATTERN_SEPARATOR = "pattern-separator";
    public final static String PERCENT = "percent";
    public final static String PER_MILLE = "per-mille";
    public final static String PRIORITY = "priority";
    public final static String REGEX = "regex";
    public final static String REQUIRED = "required";
    public final static String RESULT_PREFIX = "result-prefix";
    public final static String RULES = "rules";
    public final static String SCHEMA_LOCATION = "schema-location";
    public final static String SELECT = "select";
    public final static String SEPARATOR = "separator";
    public final static String SRC = "src";
    public final static String STANDALONE = "standalone";
    public final static String STRENGTH = "strength";
    public final static String STRING = "string";
    public final static String STYLESHEET_PREFIX = "stylesheet-prefix";
    public final static String TERMINATE = "terminate";
    public final static String TEST = "test";
    public final static String TUNNEL = "tunnel";
    public final static String TYPE = "type";
    public final static String UNDECLARE_NAMESPACES = "undeclare-namespaces";
    public final static String USE = "use";
    public final static String USE_ATTRIBUTE_SETS = "use-attribute-sets";
    public final static String USE_CHARACTER_MAPS = "use-character-maps";
    public final static String VALIDATION = "validation";
    public final static String VALUE = "value";
    public final static String VERSION = "version";
    public final static String XPATH_DEFAULT_NAMESPACE = "xpath-default-namespace";
    public final static String ZERO_DIGIT = "zero-digit";

    public final static int XS_STRING          = XS +  1;
    public final static int XS_BOOLEAN         = XS +  2;
    public final static int XS_DECIMAL         = XS +  3;
    public final static int XS_FLOAT           = XS +  4;
    public final static int XS_DOUBLE          = XS +  5;
    public final static int XS_DURATION        = XS +  6;
    public final static int XS_DATE_TIME       = XS +  7;
    public final static int XS_TIME            = XS +  8;
    public final static int XS_DATE            = XS +  9;
    public final static int XS_G_YEAR_MONTH    = XS +  10;
    public final static int XS_G_YEAR          = XS +  11;
    public final static int XS_G_MONTH_DAY     = XS +  12;
    public final static int XS_G_DAY           = XS +  13;
    public final static int XS_G_MONTH         = XS +  14;
    public final static int XS_HEX_BINARY      = XS +  15;
    public final static int XS_BASE64_BINARY   = XS +  16;
    public final static int XS_ANY_URI         = XS +  17;
    public final static int XS_QNAME           = XS +  18;
    public final static int XS_NOTATION        = XS +  19;
    public final static int XS_INTEGER                 = XS +  20;

    // Note that any type code <= XS_INTEGER is considered to represent a
    // primitive type: see Type.isPrimitiveType()

    public final static int XS_NON_POSITIVE_INTEGER    = XS +  21;
    public final static int XS_NEGATIVE_INTEGER        = XS +  22;
    public final static int XS_LONG                    = XS +  23;
    public final static int XS_INT                     = XS +  24;
    public final static int XS_SHORT                   = XS +  25;
    public final static int XS_BYTE                    = XS +  26;
    public final static int XS_NON_NEGATIVE_INTEGER    = XS +  27;
    public final static int XS_POSITIVE_INTEGER        = XS +  28;
    public final static int XS_UNSIGNED_LONG           = XS +  29;
    public final static int XS_UNSIGNED_INT            = XS +  30;
    public final static int XS_UNSIGNED_SHORT          = XS +  31;
    public final static int XS_UNSIGNED_BYTE           = XS +  32;
    public final static int XS_NORMALIZED_STRING       = XS +  41;
    public final static int XS_TOKEN                   = XS +  42;
    public final static int XS_LANGUAGE                = XS +  43;
    public final static int XS_NMTOKEN                 = XS +  44;
    public final static int XS_NMTOKENS                = XS +  45;      // NB: list type
    public final static int XS_NAME                    = XS +  46;
    public final static int XS_NCNAME                  = XS +  47;
    public final static int XS_ID                      = XS +  48;
    public final static int XS_IDREF                   = XS +  49;
    public final static int XS_IDREFS                  = XS +  50;      // NB: list type
    public final static int XS_ENTITY                  = XS +  51;
    public final static int XS_ENTITIES                = XS +  52;      // NB: list type

    public final static int XS_ANY_TYPE                = XS +  80;
    public final static int XS_ANY_SIMPLE_TYPE         = XS +  81;

    public final static int XS_INVALID_NAME             = XS +  82;

    public static final int XDT_UNTYPED                 = XDT + 1;
    public static final int XDT_UNTYPED_ATOMIC          = XDT + 2;
    public static final int XDT_ANY_ATOMIC_TYPE         = XDT + 3;
    public static final int XDT_YEAR_MONTH_DURATION     = XDT + 4;
    public static final int XDT_DAY_TIME_DURATION       = XDT + 5;
    public static final int XDT_NUMERIC                 = XDT + 6;

    public static final int XSI_TYPE                    = XSI + 1;
    public static final int XSI_NIL                     = XSI + 2;
    public static final int XSI_SCHEMA_LOCATION         = XSI + 3;
    public static final int XSI_NO_NAMESPACE_SCHEMA_LOCATION = XSI + 4;


    private static String[] localNames = new String[1023];
    private static HashMap lookup = new HashMap(1023);

    private StandardNames() {
    	//pool = namePool;
    }

    private static void bindXSLTName(int constant, String localName) {
        localNames[constant] = localName;
        lookup.put("{" + NamespaceConstant.XSLT + "}" + localName, new Integer(constant));
    }

    private static void bindSaxonName(int constant, String localName) {
        localNames[constant] = localName;
        lookup.put("{" + NamespaceConstant.SAXON + "}" + localName, new Integer(constant));
    }

    private static void bindXMLName(int constant, String localName) {
        localNames[constant] = localName;
        lookup.put("{" + NamespaceConstant.XML + "}" + localName, new Integer(constant));
    }

//    private static void bindXSLTAttributeName(int constant, String localName) {
//        localNames[constant] = localName;
//        lookup.put("{}" + localName, new Integer(constant));
//    }

    private static void bindXSName(int constant, String localName) {
        localNames[constant] = localName;
        lookup.put("{" + NamespaceConstant.SCHEMA + "}" + localName, new Integer(constant));
    }

    private static void bindXDTName(int constant, String localName) {
        localNames[constant] = localName;
        lookup.put("{" + NamespaceConstant.XDT + "}" + localName, new Integer(constant));
    }

    private static void bindXSIName(int constant, String localName) {
        localNames[constant] = localName;
        lookup.put("{" + NamespaceConstant.SCHEMA_INSTANCE + "}" + localName, new Integer(constant));
    }

	static {

		bindXSLTName(XSL_ANALYZE_STRING, "analyze-string");
		bindXSLTName(XSL_APPLY_IMPORTS, "apply-imports");
		bindXSLTName(XSL_APPLY_TEMPLATES, "apply-templates");
		bindXSLTName(XSL_ATTRIBUTE, "attribute");
		bindXSLTName(XSL_ATTRIBUTE_SET, "attribute-set" );
		bindXSLTName(XSL_CALL_TEMPLATE, "call-template");
		bindXSLTName(XSL_CHARACTER_MAP, "character-map");
		bindXSLTName(XSL_CHOOSE, "choose");
		bindXSLTName(XSL_COMMENT , "comment");
		bindXSLTName(XSL_COPY, "copy");
		bindXSLTName(XSL_COPY_OF, "copy-of");
		bindXSLTName(XSL_DECIMAL_FORMAT, "decimal-format");
		bindXSLTName(XSL_DOCUMENT, "document");
		bindXSLTName(XSL_ELEMENT, "element");
		bindXSLTName(XSL_FALLBACK, "fallback");
		bindXSLTName(XSL_FOR_EACH, "for-each");
		bindXSLTName(XSL_FOR_EACH_GROUP, "for-each-group");
		bindXSLTName(XSL_FUNCTION, "function");
		bindXSLTName(XSL_IF, "if");
		bindXSLTName(XSL_IMPORT, "import");
		bindXSLTName(XSL_IMPORT_SCHEMA, "import-schema");
		bindXSLTName(XSL_INCLUDE, "include");
		bindXSLTName(XSL_KEY, "key");
		bindXSLTName(XSL_MATCHING_SUBSTRING, "matching-substring");
		bindXSLTName(XSL_MESSAGE, "message");
		bindXSLTName(XSL_NEXT_MATCH, "next-match");
		bindXSLTName(XSL_NUMBER, "number");
		bindXSLTName(XSL_NAMESPACE, "namespace");
		bindXSLTName(XSL_NAMESPACE_ALIAS, "namespace-alias");
		bindXSLTName(XSL_NON_MATCHING_SUBSTRING, "non-matching-substring");
		bindXSLTName(XSL_OTHERWISE, "otherwise");
		bindXSLTName(XSL_OUTPUT, "output");
		bindXSLTName(XSL_OUTPUT_CHARACTER, "output-character");
		bindXSLTName(XSL_PARAM, "param");
		bindXSLTName(XSL_PERFORM_SORT, "perform-sort");
		bindXSLTName(XSL_PRESERVE_SPACE, "preserve-space");
		bindXSLTName(XSL_PROCESSING_INSTRUCTION, "processing-instruction");
        bindXSLTName(XSL_RESULT_DOCUMENT, "result-document");
		bindXSLTName(XSL_SEQUENCE, "sequence");
		bindXSLTName(XSL_SORT, "sort");
		bindXSLTName(XSL_STRIP_SPACE, "strip-space");
		bindXSLTName(XSL_STYLESHEET, "stylesheet");
		bindXSLTName(XSL_TEMPLATE, "template");
		bindXSLTName(XSL_TEXT, "text");
		bindXSLTName(XSL_TRANSFORM, "transform");
		bindXSLTName(XSL_VALUE_OF, "value-of");
		bindXSLTName(XSL_VARIABLE, "variable");
		bindXSLTName(XSL_WITH_PARAM, "with-param");
		bindXSLTName(XSL_WHEN, "when");

        bindXSLTName(XSL_XPATH_DEFAULT_NAMESPACE, "xpath-default-namespace");
        bindXSLTName(XSL_EXCLUDE_RESULT_PREFIXES, "exclude-result-prefixes");
        bindXSLTName(XSL_EXTENSION_ELEMENT_PREFIXES, "extension-element-prefixes");
        bindXSLTName(XSL_TYPE, "type");
        bindXSLTName(XSL_USE_ATTRIBUTE_SETS, "use-attribute-sets");
        bindXSLTName(XSL_VALIDATION, "validation");
        bindXSLTName(XSL_VERSION, "version");


		bindSaxonName(SAXON_ASSIGN,  "assign");
		bindSaxonName(SAXON_CALL_TEMPLATE,  "call-template");
		bindSaxonName(SAXON_COLLATION,  "collation");
		bindSaxonName(SAXON_DOCTYPE,  "doctype");
		bindSaxonName(SAXON_ENTITY_REF,  "entity-ref");
		bindSaxonName(SAXON_IMPORT_QUERY,  "import-query");
        bindSaxonName(SAXON_SCRIPT,  "script");
		bindSaxonName(SAXON_WHILE,  "while");
		//bindSaxonName(SAXON_ASSIGNABLE, "assignable");
        //bindSaxonName(SAXON_EXPLAIN, "explain");
        //bindSaxonName(SAXON_INDENT_SPACES, "indent-spaces");
        //bindSaxonName(SAXON_CHARACTER_REPRESENTATION, "character-representation");
		//bindSaxonName(SAXON_NEXT_IN_CHAIN, "next-in-chain");
        //bindSaxonName(SAXON_BYTE_ORDER_MARK, "byte-order-mark");
        //bindSaxonName(SAXON_REQUIRE_WELL_FORMED, "require-well-formed");
        //bindSaxonName(SAXON_MEMO_FUNCTION, "memo-function");

        bindXMLName(XML_BASE, "base");
        bindXMLName(XML_SPACE, "space");
        bindXMLName(XML_LANG, "lang");

//		bindXSLTAttributeName(ARCHIVE, "archive");
//		bindXSLTAttributeName(AS, "as");
//		bindXSLTAttributeName(CASE_ORDER, "case-order");
//		bindXSLTAttributeName(CDATA_SECTION_ELEMENTS, "cdata-section-elements");
//		bindXSLTAttributeName(CHARACTER, "character");
//		bindXSLTAttributeName(CLASS, "class");
//		bindXSLTAttributeName(COLLATION, "collation");
//		bindXSLTAttributeName(COPY_NAMESPACES, "copy-namespaces");
//		bindXSLTAttributeName(COUNT, "count");
//		bindXSLTAttributeName(DATA_TYPE, "data-type");
//		bindXSLTAttributeName(DECIMAL_SEPARATOR, "decimal-separator");
//		bindXSLTAttributeName(DEFAULT, "default");
//		bindXSLTAttributeName(DEFAULT_VALIDATION, "default-validation");
//		bindXSLTAttributeName(DECOMPOSITION, "decomposition");
//		bindXSLTAttributeName(DIGIT, "digit");
//		bindXSLTAttributeName(DISABLE_OUTPUT_ESCAPING, "disable-output-escaping");
//		bindXSLTAttributeName(DOCTYPE_PUBLIC, "doctype-public");
//		bindXSLTAttributeName(DOCTYPE_SYSTEM, "doctype-system");
//		bindXSLTAttributeName(ELEMENTS, "elements");
//		bindXSLTAttributeName(ENCODING, "encoding");
//        bindXSLTAttributeName(ESCAPE_URI_ATTRIBUTES, "escape-uri-attributes");
//		bindXSLTAttributeName(EXCLUDE_RESULT_PREFIXES, "exclude-result-prefixes");
//		bindXSLTAttributeName(EXTENSION_ELEMENT_PREFIXES, "extension-element-prefixes");
//		bindXSLTAttributeName(FLAGS, "flags");
//		bindXSLTAttributeName(FORMAT, "format");
//		bindXSLTAttributeName(FROM, "from");
//		bindXSLTAttributeName(GROUPING_SEPARATOR, "grouping-separator");
//		bindXSLTAttributeName(GROUPING_SIZE, "grouping-size");
//		bindXSLTAttributeName(GROUP_ADJACENT, "group-adjacent");
//		bindXSLTAttributeName(GROUP_BY, "group-by");
//		bindXSLTAttributeName(GROUP_ENDING_WITH, "group-ending-with");
//		bindXSLTAttributeName(GROUP_STARTING_WITH, "group-starting-with");
//		bindXSLTAttributeName(HREF, "href");
//		bindXSLTAttributeName(ID, "id");
//        bindXSLTAttributeName(INCLUDE_CONTENT_TYPE, "include-content-type");
//		bindXSLTAttributeName(IMPLEMENTS_PREFIX, "implements-prefix");
//		bindXSLTAttributeName(INDENT, "indent");
//		bindXSLTAttributeName(INFINITY, "infinity");
//		bindXSLTAttributeName(LANG, "lang");
//		bindXSLTAttributeName(LANGUAGE, "language");
//		bindXSLTAttributeName(LETTER_VALUE, "letter-value");
//		bindXSLTAttributeName(LEVEL, "level");
//		bindXSLTAttributeName(MATCH, "match");
//		bindXSLTAttributeName(MEDIA_TYPE, "media-type");
//		bindXSLTAttributeName(METHOD, "method");
//		bindXSLTAttributeName(MINUS_SIGN, "minus-sign");
//		bindXSLTAttributeName(MODE, "mode");
//		bindXSLTAttributeName(NAME, "name");
//		bindXSLTAttributeName(NAMESPACE, "namespace");
//		bindXSLTAttributeName(NAN, "NaN");
//		bindXSLTAttributeName(OMIT_XML_DECLARATION, "omit-xml-declaration");
//		bindXSLTAttributeName(ORDER, "order");
//		bindXSLTAttributeName(ORDINAL, "ordinal");
//		bindXSLTAttributeName(OVERRIDE, "override");
//		bindXSLTAttributeName(PATTERN_SEPARATOR, "pattern-separator");
//		bindXSLTAttributeName(PERCENT, "percent");
//		bindXSLTAttributeName(PER_MILLE, "per-mille");
//		bindXSLTAttributeName(PRIORITY, "priority");
//		bindXSLTAttributeName(REGEX, "regex");
//		bindXSLTAttributeName(REQUIRED, "required");
//		bindXSLTAttributeName(RESULT_PREFIX, "result-prefix");
//		bindXSLTAttributeName(RULES, "rules");
//		bindXSLTAttributeName(SCHEMA_LOCATION, "schema-location");
//		bindXSLTAttributeName(SELECT, "select");
//		bindXSLTAttributeName(SEPARATOR, "separator");
//		bindXSLTAttributeName(SRC, "src");
//		bindXSLTAttributeName(STANDALONE, "standalone");
//		bindXSLTAttributeName(STRENGTH, "strength");
//		bindXSLTAttributeName(STRING, "string");
//		bindXSLTAttributeName(STYLESHEET_PREFIX, "stylesheet-prefix");
//		bindXSLTAttributeName(TERMINATE, "terminate");
//		bindXSLTAttributeName(TEST, "test");
//		bindXSLTAttributeName(TUNNEL, "tunnel");
//		bindXSLTAttributeName(TYPE, "type");
//        bindXSLTAttributeName(UNDECLARE_NAMESPACES, "undeclare-namespaces");
//		bindXSLTAttributeName(USE, "use");
//		bindXSLTAttributeName(USE_ATTRIBUTE_SETS, "use-attribute-sets");
//		bindXSLTAttributeName(USE_CHARACTER_MAPS, "use-character-maps");
//		bindXSLTAttributeName(VALIDATION, "validation");
//		bindXSLTAttributeName(VALUE, "value");
//		bindXSLTAttributeName(VERSION, "version");
//		bindXSLTAttributeName(ZERO_DIGIT, "zero-digit");
//        bindXSLTAttributeName(XPATH_DEFAULT_NAMESPACE, "xpath-default-namespace");

        bindXSName(XS_STRING, "string");
        bindXSName(XS_BOOLEAN, "boolean");
        bindXSName(XS_DECIMAL, "decimal");
        bindXSName(XS_FLOAT, "float");
        bindXSName(XS_DOUBLE, "double");
        bindXSName(XS_DURATION, "duration");
        bindXSName(XS_DATE_TIME, "dateTime");
        bindXSName(XS_TIME, "time");
        bindXSName(XS_DATE, "date");
        bindXSName(XS_G_YEAR_MONTH , "gYearMonth");
        bindXSName(XS_G_YEAR, "gYear");
        bindXSName(XS_G_MONTH_DAY, "gMonthDay");
        bindXSName(XS_G_DAY, "gDay");
        bindXSName(XS_G_MONTH, "gMonth");
        bindXSName(XS_HEX_BINARY, "hexBinary");
        bindXSName(XS_BASE64_BINARY, "base64Binary");
        bindXSName(XS_ANY_URI, "anyURI");
        bindXSName(XS_QNAME, "QName");
        bindXSName(XS_NOTATION, "NOTATION");
        bindXSName(XS_INTEGER, "integer");
        bindXSName(XS_NON_POSITIVE_INTEGER, "nonPositiveInteger");
        bindXSName(XS_NEGATIVE_INTEGER, "negativeInteger");
        bindXSName(XS_LONG, "long");
        bindXSName(XS_INT, "int");
        bindXSName(XS_SHORT, "short");
        bindXSName(XS_BYTE, "byte");
        bindXSName(XS_NON_NEGATIVE_INTEGER, "nonNegativeInteger");
        bindXSName(XS_POSITIVE_INTEGER, "positiveInteger");
        bindXSName(XS_UNSIGNED_LONG, "unsignedLong");
        bindXSName(XS_UNSIGNED_INT, "unsignedInt");
        bindXSName(XS_UNSIGNED_SHORT, "unsignedShort");
        bindXSName(XS_UNSIGNED_BYTE, "unsignedByte");
        bindXSName(XS_NORMALIZED_STRING, "normalizedString");
        bindXSName(XS_TOKEN, "token");
        bindXSName(XS_LANGUAGE, "language");
        bindXSName(XS_NMTOKEN, "NMTOKEN");
        bindXSName(XS_NMTOKENS, "NMTOKENS");      // NB: list type
        bindXSName(XS_NAME, "Name");
        bindXSName(XS_NCNAME, "NCName");
        bindXSName(XS_ID, "ID");
        bindXSName(XS_IDREF, "IDREF");
        bindXSName(XS_IDREFS, "IDREFS");      // NB: list type
        bindXSName(XS_ENTITY, "ENTITY");
        bindXSName(XS_ENTITIES, "ENTITIES");      // NB: list type

        bindXSName(XS_ANY_TYPE, "anyType");
        bindXSName(XS_ANY_SIMPLE_TYPE, "anySimpleType");
        bindXSName(XS_INVALID_NAME, "invalidName");

        bindXDTName(XDT_UNTYPED, "untyped");
        bindXDTName(XDT_UNTYPED_ATOMIC, "untypedAtomic");
        bindXDTName(XDT_ANY_ATOMIC_TYPE, "anyAtomicType");
        bindXDTName(XDT_YEAR_MONTH_DURATION, "yearMonthDuration");
        bindXDTName(XDT_DAY_TIME_DURATION, "dayTimeDuration");
        bindXDTName(XDT_NUMERIC, "numeric");

        bindXSIName(XSI_TYPE, "type");
        bindXSIName(XSI_NIL, "nil");
        bindXSIName(XSI_SCHEMA_LOCATION, "schemaLocation");
        bindXSIName(XSI_NO_NAMESPACE_SCHEMA_LOCATION, "noNamespaceSchemaLocation");
	}

    public static int getFingerprint(String uri, String localName) {
        Integer fp = (Integer)lookup.get("{" + uri + "}" + localName);
        if (fp==null) {
            return -1;
        } else {
            return fp.intValue();
        }
    }

    public static String getLocalName(int fingerprint) {
        return localNames[fingerprint];
    }

    public static String getURI(int fingerprint) {
        int c = fingerprint >> 7;
        switch (c) {
            case DFLT_NS: return "";
            case XSL_NS: return NamespaceConstant.XSLT;
            case SAXON_NS: return NamespaceConstant.SAXON;
            case XML_NS: return NamespaceConstant.XML;
            case XS_NS: return NamespaceConstant.SCHEMA;
            case XSI_NS: return NamespaceConstant.SCHEMA_INSTANCE;
            case XDT_NS: return NamespaceConstant.XDT;
            default: return null;
        }
    }

    public static short getURICode(int fingerprint) {
        int c = fingerprint >> 7;
        switch (c) {
            case DFLT_NS: return 0;
            case XSL_NS: return NamespaceConstant.XSLT_CODE;
            case SAXON_NS: return NamespaceConstant.SAXON_CODE;
            case XML_NS: return NamespaceConstant.XML_CODE;
            case XS_NS: return NamespaceConstant.SCHEMA_CODE;
            case XSI_NS: return NamespaceConstant.XSI_CODE;
            case XDT_NS: return NamespaceConstant.XDT_CODE;
            default: return -1;
        }
    }

    /**
     * Get the Clark form of a name, given its name code or fingerprint
     * @return the local name if the name is in the null namespace, or "{uri}local"
     * otherwise. The name is always interned.
     */

    public static String getClarkName(int fingerprint) {
        String uri = getURI(fingerprint);
        if (uri.equals("")) {
            return getLocalName(fingerprint);
        } else {
            return "{" + uri + "}" + getLocalName(fingerprint);
        }
    }

    public static String getPrefix(int fingerprint) {
        int c = fingerprint >> 7;
        switch (c) {
            case DFLT_NS: return "";
            case XSL_NS: return "xsl";
            case SAXON_NS: return "saxon";
            case XML_NS: return "xml";
            case XS_NS: return "xs";
            case XSI_NS: return "xsi";
            case XDT_NS: return "xdt";
            default: return null;
        }
    }

    public static String getDisplayName(int fingerprint) {
        if (fingerprint==-1) {
            return "(anonymous type)";
        }
        if ((fingerprint>>7) == DFLT) {
            return getLocalName(fingerprint);
        }
        return getPrefix(fingerprint) + ":" + getLocalName(fingerprint);
    }

}
