package net.sf.saxon.trace;

import java.util.HashMap;

/**
* Information about an instruction in the stylesheet, made
* available at run-time to a TraceListener
*/

public interface InstructionInfo {

    /**
    * Get the name of the instruction. This will normally be the name of
    * an XSLT instruction, a literal result element, or an extension element.
    * The name is passed in a form suitable for display to the user: it will
    * generally be a QName, but this is not guaranteed. Some instructions
    * may be internally generated by the Saxon compiler, in which case the name
    * might not relate to anything obvious in the source stylesheet
    */

    public String getInstructionName();

    /**
    * Get the system identifier (URI) of the source stylesheet module containing
    * the instruction. This will generally be an absolute URI. If the system
    * identifier is not known, the method may return null. In some cases, for example
    * where XML external entities are used, the correct system identifier is not
    * always retained.
    */

    public String getSystemId();

    /**
    * Get the line number of the instruction in the source stylesheet module.
    * If this is not known, or if the instruction is an artificial one that does
    * not relate to anything in the source code, the value returned may be -1.
    */

    public int getLineNumber();

    /**
     * Get the value of a particular property of the instruction. Properties
     * of XSLT instructions are generally known by the name of the stylesheet attribute
     * that defines them.
     * @param name The name of the required property
     * @return  The value of the requested property, or null if the property is not available
     */

    public Object getProperty(String name);

    /**
     * Get all the properties of the instruction. Properties
     * of XSLT instructions are generally known by the name of the stylesheet attribute
     * that defines them.
     * @return a HashMap containing all the properties of this instruction.
     */

    public HashMap getProperties();

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of Software AG (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
