package net.sf.saxon.tree;
import net.sf.saxon.om.NodeInfo;
import net.sf.saxon.event.Receiver;
import net.sf.saxon.type.Type;

import org.w3c.dom.Text;
import javax.xml.transform.TransformerException;

/**
  * A node in the XML parse tree representing character content<P>
  * @author <A HREF="mailto:michael.h.kay@ntlworld.com>Michael H. Kay</A>
  */

final class TextImpl extends NodeImpl implements Text {

    private String content;

    public TextImpl(ParentNodeImpl parent, String content) {
    	this.parent = parent;
    	this.content = content;
    }

    /**
    * Return the character value of the node.
    * @return the string value of the node
    */

    public String getStringValue() {
		return content;
    }

    /**
    * Return the type of node.
    * @return Type.TEXT
    */

    public final int getNodeKind() {
        return Type.TEXT;
    }

    /**
    * Copy this node to a given outputter
    */

    public void copy(Receiver out, int whichNamespaces) throws TransformerException {
        out.characters(content, 0);
    }

    /**
    * Copy the string-value of this node to a given outputter
    */

    //public void copyStringValue(Receiver out) throws TransformerException {
    //    out.characters(content, 0);
    //}

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//