/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio, Inc.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio, Inc. Exolab is a registered
 *    trademark of Intalio, Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * INTALIO, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2002 (C) Intalio Inc. All Rights Reserved.
 *
 * $Id: AnyType.java,v 1.1 2004/03/09 10:10:35 mhkay Exp $
 */

package net.sf.saxon.type;

import net.sf.saxon.om.SequenceIterator;
import net.sf.saxon.om.NodeInfo;
import net.sf.saxon.om.SingletonIterator;
import net.sf.saxon.xpath.XPathException;
import net.sf.saxon.value.UntypedAtomicValue;

/**
 * A Class which represents the XML Schema AnyType.
 * <BR />
 *
 * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
 * @version $Revision: 1.1 $ $Date: 2004/03/09 10:10:35 $
**/

public final class AnyType extends SchemaType {

    private static AnyType theInstance = new AnyType();

    /**
     * Private constructor
     */
    private AnyType() {
        super();
    }

    /**
     * Get the singular instance of this class
     * @return the singular object representing xs:anyType
     */

    public static AnyType getInstance() {
        return theInstance;
    }

    /**
     * Returns true if this SchemaType is a SimpleType
     * @return false
    **/
    public boolean isSimpleType() {
        return false;
    }

    /**
     * Returns the name of this type. This method
     * always returns 'anyType'.
     *
     * @return the name of this type.
     */
    public String getLocalName() {
        return "anyType";
    } //-- getName

    /**
     * Get a description of this type for use in diagnostics
     */

    public String getDescription() {
        return "xs:anyType";
    }

    /**
     * Checks the validity of this Schema defintion.
     *
     * @exception net.sf.saxon.type.SchemaException when this Schema definition
     * is invalid.
     */
    public boolean validate(javax.xml.transform.ErrorListener errorListener) throws SchemaException
    {
        return true;
         //-- do nothing, this type is always valid.
    } //-- validate

    /**
     * Get the typed value of a node that is annotated with this schema type
     */

    public SequenceIterator getTypedValue(NodeInfo node)
            throws XPathException {
        return SingletonIterator.makeIterator(
                new UntypedAtomicValue(node.getStringValue()));
    }

} //-- AnyType
