package net.sf.saxon.type;

import net.sf.saxon.value.AtomicValue;
import net.sf.saxon.value.StringValue;
import net.sf.saxon.type.Type;
import net.sf.saxon.xpath.XPathException;

/**
 * Represents the minLength facet
 */
public class MinLengthFacet extends LengthFacet {

    public String getName() {
        return "minLength";
    }

    /**
     * Check that this facet is legal when used on a type derived by restriction
     * @param base the type from which the restricted type is derived
     * @throws SchemaException if the facet is not legal
     */

    public void checkFacetRestriction(SimpleType base) throws SchemaException {
        Facet baseFacet = base.getFacet(Facet.LENGTH);
        if (baseFacet != null) {
            if (requiredLength != baseFacet.toLong()) {
                throw new SchemaException("In a type derived by restriction, " +
                                          "the minLength facet conflicts with the length of the base type");
            }
        }
        baseFacet = base.getFacet(Facet.MIN_LENGTH);
        if (baseFacet != null) {
            if (requiredLength < baseFacet.toLong()) {
                throw new SchemaException("In a type derived by restriction, " +
                                          "the minLength facet is shorter than the minLength of the base type");
            }
        }
        baseFacet = base.getFacet(Facet.MAX_LENGTH);
        if (baseFacet != null) {
            if (requiredLength > baseFacet.toLong()) {
                throw new SchemaException("In a type derived by restriction, " +
                                          "the minLength facet is longer than the maxLength of the base type");
            }
        }
    }


    /**
     * Test whether a supplied length conforms to this required length (used
     * when testing the length of a list)
     */

    public boolean testLength(int actualLength) {
        return actualLength >= requiredLength;
    }
}
//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of Software AG (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
