package net.sf.saxon.type;

import net.sf.saxon.type.AtomicType;
import net.sf.saxon.type.Facet;
import net.sf.saxon.value.AtomicValue;
import net.sf.saxon.value.StringValue;
import net.sf.saxon.xpath.XPathException;

import java.util.regex.Pattern;

/**
 * Represents a pattern facet on a schema-defined simple type
 */
public class PatternFacet extends Facet {

    StringValue requiredValue;
    Pattern regex;

    public PatternFacet() {
    }

    public String getName() {
        return Facet.PATTERN;
    }

    public String getValue() {
        return requiredValue.getStringValue();
    }

    public void setValue(String value) throws SchemaException {
        requiredValue = new StringValue(value);
        try {
            regex = Pattern.compile(RegexTranslator.translate(value, false));
        } catch (RegexTranslator.RegexSyntaxException e) {
            throw new SchemaException(e.getMessage());
        }
    }

    /**
      * Check that this facet is legal when used on a type derived by restriction
      * @param base the type from which the restricted type is derived
      * @throws SchemaException if the facet is not legal
      */

     public void checkFacetRestriction(SimpleType base) throws SchemaException {
//        if (!(base instanceof AtomicType)) {
//            throw new SchemaException("The " + getName() +
//                                      " facet is available only when restricting an atomic type");
//        }
     }


    /**
     * Test whether an atomic value conforms to this facet
     * @param value  the value to be tested
     * @return true if the value conforms; false if it doesn't conform, or if the test
     * fails
     */

    public boolean testAtomicValue(AtomicValue value) {
        try {
            return regex.matcher(value.getStringValue()).matches();
        } catch (XPathException err) {
            return false;
        }

    }

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of Software AG (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
