package net.sf.saxon.value;

import net.sf.saxon.xpath.XPathException;
import net.sf.saxon.type.Type;
import net.sf.saxon.style.StandardNames;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.Calendar;

/**
 * Implementation of the xs:gYear data type
 */

public class GYearValue extends DateValue {

    private static Pattern regex =
            Pattern.compile("(-?[0-9]+)(Z|[+-][0-9][0-9]:[0-9][0-9])?");

    public GYearValue(){};

    public GYearValue(CharSequence value) throws XPathException {
        Matcher m = regex.matcher(value);
        if (!m.matches()) {
            throw new XPathException.Dynamic("Cannot convert '" + value +
                    "' to a gYear");
        }
        String base = m.group(1);
        String tz = m.group(2);
        String date = base + "-01-01" + (tz==null ? "" : tz);
        setLexicalValue(date);
    }

    /**
    * Convert to target data type
    * @param requiredType an integer identifying the required atomic type
    * @return an AtomicValue, a value of the required type
    * @throws XPathException if the conversion is not possible
    */

    public AtomicValue convert(int requiredType) throws XPathException {
        switch(requiredType) {
        case Type.G_YEAR:
        case Type.ATOMIC:
        case Type.ITEM:
            return this;

        case Type.STRING:
            return new StringValue(getStringValue());
        case Type.UNTYPED_ATOMIC:
            return new UntypedAtomicValue(getStringValue());
        default:
            throw new XPathException.Dynamic("Cannot convert gYear to " +
                                     StandardNames.getDisplayName(requiredType));
        }
    }

    public String getStringValue() {

        StringBuffer sb = new StringBuffer();
        int year = calendar.get(Calendar.YEAR);
        if (year<0) {
            sb.append('-');
            year = -year;
        }
        DateTimeValue.appendString(sb, year, (year>9999 ? (calendar.get(Calendar.YEAR)+"").length() : 4));

        if (zoneSpecified) {
            DateTimeValue.appendTimezone(tzOffset, sb);
        }

        return sb.toString();

    }
}
