package net.sf.saxon.xpath;

import javax.xml.transform.TransformerException;
import javax.xml.transform.SourceLocator;

/**
* XPathException is used to indicate an error in an XPath expression.
* We don't distinguish compile-time errors from run-time errors because there are
* too many overlaps, e.g. constant expressions can be evaluated at compile-time, and
* expressions can be optimised either at compile-time or at run-time.
*/

public abstract class XPathException extends TransformerException {

    public XPathException(String message) {
        super(message);
    }

    public XPathException(Throwable err) {
        super(err);
    }

    public XPathException(String message, Throwable err) {
        super(message, err);
    }

    public XPathException(String message, SourceLocator loc) {
        super(message, loc);
    }

    /**
    * Subclass used for static errors
    */

    public static class Static extends XPathException {

        public Static(String message) {
            super(message);
        }

        public Static(Exception err) {
            super(err);
        }

        public Static(String message, Throwable err) {
            super(message, err);
        }

        public Static(String message, SourceLocator loc) {
            super(message, loc);
        }

    }

    /**
    * Subclass used for dynamic errors
    */

    public static class Dynamic extends XPathException {

        public Dynamic(String message) {
            super(message);
        }

        public Dynamic(Throwable err) {
            super(err);
        }

        public Dynamic(String message, Throwable err) {
            super(message, err);
        }

        public Dynamic(String message, SourceLocator loc) {
            super(message, loc);
        }

    }

    /**
     * Subclass used to report circularities
     */

    public static class Circularity extends Dynamic {
        public Circularity(String message) {
            super(message);
        }
    }


    /**
    * Subclass used for type errors
    */

    public static class Type extends XPathException {

        public Type(String message) {
            super(message);
        }

        public Type(Throwable err) {
            super(err);
        }

        public Type(String message, Throwable err) {
            super(message, err);
        }

        public Type(String message, SourceLocator loc) {
            super(message, loc);
        }


    }


}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License.
//
// The Original Code is: all this file.
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (michael.h.kay@ntlworld.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
//
// Contributor(s): none.
//
